@@ -5,7 +5,9 @@ use crate::{RadResult, RadError};
 #[cfg(feature = "hook")]
 pub struct HookMap {
     macro_hook : HashMap<String,HookState>,
-    char_hook : HashMap<char,HookState>
+    char_hook : HashMap<char,HookState>,
+    // This is called upon every several letters
+    letter_hook : Option<HookState>,
 }
 
 impl HookMap {
@@ -13,6 +15,7 @@ impl HookMap {
         Self {
             macro_hook: HashMap::new(),
             char_hook: HashMap::new(),
+            letter_hook: None,
         }
     }
 
@@ -50,6 +53,22 @@ impl HookMap {
         None
     }
 
+    pub fn add_letter_count(&mut self) -> Option<String> {
+        if let Some(hook_state) = &mut self.letter_hook {
+            if hook_state.enabled {
+                hook_state.current_count += 1; 
+                if hook_state.current_count == hook_state.target_count {
+                    hook_state.current_count = 0; // reset count
+                    if !hook_state.resetable { 
+                        hook_state.enabled = false;
+                    }
+                    return Some(hook_state.target_macro.clone());
+                } 
+            }
+        }
+        None
+    }
+
     pub fn switch_hook(&mut self, hook_type : HookType, index: &str, switch: bool) -> RadResult<()> {
         match hook_type {
             HookType::Macro => {
@@ -64,6 +83,11 @@ impl HookMap {
                 if let Some(state) = self.char_hook.get_mut(&index_char) {
                     state.enabled = switch
                 } else { return Err(RadError::HookMacroFail(format!("Hook trigger \"{}\" is not registered as character hook", index))) }
+            },
+            HookType::Letter => {
+                if let Some(hook_state) = &mut self.letter_hook { 
+                    hook_state.enabled = switch; 
+                }
             }
         };
         Ok(())
@@ -72,22 +96,24 @@ impl HookMap {
     pub fn add_hook(&mut self, hook_type : HookType, target: &str, invoke_macro: &str,target_count: usize, resetable : bool) -> RadResult<()> {
         let hook_state = HookState::new(invoke_macro.to_owned(),target_count,resetable);
         match hook_type {
-            HookType::Macro => self.macro_hook.insert(target.to_owned(),hook_state),
+            HookType::Macro => {self.macro_hook.insert(target.to_owned(),hook_state);}
             HookType::Char => {
                 let index_char = if let Some(ch) = target.chars().next() { ch } else {  return Err(RadError::HookMacroFail("Index is empty".to_owned())) };
-                self.char_hook.insert(index_char,hook_state)
+                self.char_hook.insert(index_char,hook_state);
             },
+            HookType::Letter => { self.letter_hook.replace(hook_state); } 
         };
         Ok(())
     }
 
     pub fn del_hook(&mut self, hook_type : HookType, index: &str) -> RadResult<()> {
         match hook_type {
-            HookType::Char => self.macro_hook.remove(index),
+            HookType::Char => {self.macro_hook.remove(index);}
             HookType::Macro => {
                 let index_char = if let Some(ch) = index.chars().next() { ch } else {  return Err(RadError::HookMacroFail("Index is empty".to_owned())) };
-                self.char_hook.remove(&index_char)
-            }
+                self.char_hook.remove(&index_char);
+            },
+            HookType::Letter => { self.letter_hook = None; }
         };
         Ok(())
     }
@@ -98,6 +124,7 @@ impl HookMap {
 pub enum HookType {
     Macro,
     Char,
+    Letter,
 }
 
 impl HookType {
@@ -105,6 +132,7 @@ impl HookType {
         let var = match hook_type.to_lowercase().as_str() {
             "macro" => Self::Macro,
             "char" => Self::Char,
+            "letter" => Self::Letter,
             _ => return Err(RadError::InvalidConversion(format!("Invalid hook type \"{}\"", hook_type)))
         };
 
