@@ -3,35 +3,36 @@ use std::iter::FromIterator;
 use std::collections::HashMap;
 use crate::{AuthType, RadError};
 use crate::utils::Utils;
+use crate::models::RadResult;
 use crate::arg_parser::ArgParser;
 use crate::Processor;
 
-type KeywordMacType = fn(&str, usize,&mut Processor) -> Result<Option<String>, RadError>;
+type KMacroType = fn(&str, usize,&mut Processor) -> RadResult<Option<String>>;
 
 #[derive(Clone)]
-pub struct KeywordMacro {
-    macros : HashMap<String, KeywordMacType>,
+pub struct KeywordMacroMap {
+    pub(crate) macros : HashMap<&'static str, KMacroSign>,
 }
 
-impl KeywordMacro {
+impl KeywordMacroMap {
     pub fn new() -> Self {
         let map = HashMap::from_iter(IntoIter::new([
-            ("bind".to_owned(),    KeywordMacro::bind_depre       as KeywordMacType),
-            ("declare".to_owned(), KeywordMacro::declare          as KeywordMacType),
-            ("fassert".to_owned(), KeywordMacro::assert_fail      as KeywordMacType),
-            ("foreach".to_owned(), KeywordMacro::foreach          as KeywordMacType),
-            ("forloop".to_owned(), KeywordMacro::forloop          as KeywordMacType),
-            ("global".to_owned(),  KeywordMacro::global_depre     as KeywordMacType),
-            ("if".to_owned(),      KeywordMacro::if_cond          as KeywordMacType),
-            ("ifdef".to_owned(),   KeywordMacro::ifdef            as KeywordMacType),
-            ("ifdefel".to_owned(), KeywordMacro::ifdefel          as KeywordMacType),
-            ("ifelse".to_owned(),  KeywordMacro::ifelse           as KeywordMacType),
-            ("ifenv".to_owned(),   KeywordMacro::ifenv            as KeywordMacType),
-            ("ifenvel".to_owned(), KeywordMacro::ifenvel          as KeywordMacType),
-            ("let".to_owned(),     KeywordMacro::bind_to_local    as KeywordMacType),
-            ("pause".to_owned(),   KeywordMacro::pause            as KeywordMacType),
-            ("repl".to_owned(),    KeywordMacro::replace          as KeywordMacType),
-            ("static".to_owned(),  KeywordMacro::define_static    as KeywordMacType),
+            ("bind",    KMacroSign::new("bind",    ["a_macro_name","a_value"],KeywordMacroMap::bind_depre)),
+            ("declare", KMacroSign::new("declare", ["a_macro_names"],KeywordMacroMap::declare)),
+            ("fassert", KMacroSign::new("fassert", ["a_lvalue","a_rvalue"],KeywordMacroMap::assert_fail)),
+            ("foreach", KMacroSign::new("foreach", ["a_array","a_body"],KeywordMacroMap::foreach)),
+            ("forloop", KMacroSign::new("forloop", ["a_min","a_max","a_body"],KeywordMacroMap::forloop)),
+            ("global",  KMacroSign::new("global",  ["a_macro_name","a_value"],KeywordMacroMap::global_depre)),
+            ("if",      KMacroSign::new("if",      ["a_boolean","a_if_expr"],KeywordMacroMap::if_cond)),
+            ("ifelse",  KMacroSign::new("ifelse",  ["a_boolean","a_if_expr","a_else_expr"],KeywordMacroMap::ifelse)),
+            ("ifdef",   KMacroSign::new("ifdef",   ["a_macro_name","a_if_expr"],KeywordMacroMap::ifdef)),
+            ("ifdefel", KMacroSign::new("ifdefel", ["a_macro_name","a_if_expr","a_else_expr"],KeywordMacroMap::ifdefel)),
+            ("ifenv",   KMacroSign::new("ifenv",   ["a_env_name","a_if_expr"],KeywordMacroMap::ifenv)),
+            ("ifenvel", KMacroSign::new("ifenvel", ["a_env_name","a_if_expr","a_else_expr"],KeywordMacroMap::ifenvel)),
+            ("let",     KMacroSign::new("let",     ["a_macro_name","a_value"],KeywordMacroMap::bind_to_local)),
+            ("pause",   KMacroSign::new("pause",   ["a_pause?"],KeywordMacroMap::pause)),
+            ("repl",    KMacroSign::new("repl",    ["a_macro_name","a_new_value"],KeywordMacroMap::replace)),
+            ("static",  KMacroSign::new("static",  ["a_macro_name","a_value"],KeywordMacroMap::define_static)),
         ]));
         Self {
             macros: map,
@@ -39,9 +40,9 @@ impl KeywordMacro {
     }
 
     /// Get Function pointer from map
-    pub fn get(&self, name: &str) -> Option<&KeywordMacType> {
+    pub fn get_func(&self, name: &str) -> Option<&KMacroType> {
         if let Some(mac) = self.macros.get(name) {
-            Some(mac)
+            Some(&mac.logic)
         } else {
             None
         }
@@ -63,7 +64,7 @@ impl KeywordMacro {
     /// 
     /// $pause(true)
     /// $pause(false)
-    fn pause(args: &str, level: usize,processor : &mut Processor) -> Result<Option<String>, RadError> {
+    fn pause(args: &str, level: usize,processor : &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 1, true) {
             let arg = &processor.parse_chunk_args(level, "", &args[0])?;
 
@@ -89,7 +90,7 @@ impl KeywordMacro {
     /// # Usage 
     ///
     /// $foreach(\*a,b,c*\,$:)
-    fn foreach(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn foreach(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let mut sums = String::new();
             let loopable = &processor.parse_chunk_args(level, "", &args[0])?;
@@ -109,7 +110,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $forloop(1,5,$:)
-    fn forloop(args: &str, level: usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn forloop(args: &str, level: usize, processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 3, true) {
             let mut sums = String::new();
 
@@ -145,7 +146,7 @@ impl KeywordMacro {
     /// # Usage 
     ///
     /// $if(evaluation, ifstate)
-    fn if_cond(args: &str,level:usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn if_cond(args: &str,level:usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let boolean = &processor.parse_chunk_args(level,"",&args[0])?;
 
@@ -171,7 +172,7 @@ impl KeywordMacro {
     /// # Usage 
     ///
     /// $ifelse(evaluation, \*ifstate*\, \*elsestate*\)
-    fn ifelse(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn ifelse(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 3, true) {
             let boolean = &processor.parse_chunk_args(level,"",&args[0])?;
 
@@ -199,7 +200,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $ifdef(macro_name, expr)
-    fn ifdef(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn ifdef(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let name = processor.parse_chunk_args(level, "",&Utils::trim(&args[0]))?;
             let map = processor.get_map();
@@ -221,7 +222,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $ifdefelse(macro_name,expr,expr2)
-    fn ifdefel(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn ifdefel(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 3, true) {
             let name = processor.parse_chunk_args(level, "",&Utils::trim(&args[0]))?;
             let map = processor.get_map();
@@ -245,7 +246,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $ifenv(env_name, expr)
-    fn ifenv(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn ifenv(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if !Utils::is_granted("ifenv", AuthType::ENV,processor)? {
             return Ok(None);
         }
@@ -274,7 +275,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $ifenvel(env_name,expr,expr2)
-    fn ifenvel(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn ifenvel(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if !Utils::is_granted("ifenvel", AuthType::ENV,processor)? {
             return Ok(None);
         }
@@ -305,7 +306,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $repl(macro,value)
-    fn replace(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn replace(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let name = processor.parse_chunk_args(level, "",&Utils::trim(&args[0]))?;
             let target = args[1].as_str();
@@ -323,7 +324,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $fassert(abc,abc)
-    fn assert_fail(args: &str, level: usize,processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn assert_fail(args: &str, level: usize,processor: &mut Processor) -> RadResult<Option<String>> {
         let result = processor.parse_chunk_args(level, "", args);
         if let Err(_) = result {
             processor.track_assertion(true)?;
@@ -335,7 +336,7 @@ impl KeywordMacro {
     }
 
     #[deprecated(since = "1.2", note = "Bind is deprecated and will be removed in 2.0")]
-    fn bind_depre(args: &str, level:usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn bind_depre(args: &str, level:usize, processor: &mut Processor) -> RadResult<Option<String>> {
         processor.log_warning("Bind is deprecated and will be removed in 2.0 version. Use let instead.")?;
         Self::bind_to_local(args, level, processor)
     }
@@ -345,7 +346,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $declare(n1,n2,n3)
-    fn declare(args: &str, level:usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn declare(args: &str, level:usize, processor: &mut Processor) -> RadResult<Option<String>> {
         let names = processor.parse_chunk_args(level, "",&Utils::trim(args))?;
         // TODO Create empty macro rules
         let custom_rules = names
@@ -376,7 +377,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $let(name,value)
-    fn bind_to_local(args: &str, level:usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn bind_to_local(args: &str, level:usize, processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let name = processor.parse_chunk_args(level, "",&Utils::trim(&args[0]))?;
             let value = processor.parse_chunk_args(level, "",&Utils::trim(&args[1]))?;
@@ -395,7 +396,7 @@ impl KeywordMacro {
     /// This is technically same with static
     /// This macro will be completely removed in 2.0
     #[deprecated(since = "1.2", note = "Global is deprecated and will be removed in 2.0")]
-    fn global_depre(args: &str, level: usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn global_depre(args: &str, level: usize, processor: &mut Processor) -> RadResult<Option<String>> {
         processor.log_warning("Global is deprecated and will be removed in 2.0 version. Use static instead.")?;
         Self::define_static(args,level,processor)
     }
@@ -405,7 +406,7 @@ impl KeywordMacro {
     /// # Usage
     ///
     /// $static(name,value)
-    fn define_static(args: &str, level : usize, processor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn define_static(args: &str, level : usize, processor: &mut Processor) -> RadResult<Option<String>> {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
             let name = processor.parse_chunk_args(level, "",&Utils::trim(&args[0]))?;
             let value = processor.parse_chunk_args(level, "",&Utils::trim(&args[1]))?;
@@ -420,7 +421,7 @@ impl KeywordMacro {
                     processor.log_warning(&format!("Creating a static macro with a name already existing : \"{}\"", name))?;
                 }
             }
-            processor.add_custom_rules(vec![(&name,"",&value)])?;
+            processor.add_static_rules(vec![(&name,&value)])?;
             Ok(None)
         } else {
             Err(RadError::InvalidArgument("Static requires two argument".to_owned()))
@@ -430,3 +431,43 @@ impl KeywordMacro {
     // Keyword macros end
     // ----------
 }
+
+/// Keyword Macro signature
+#[derive(Clone)]
+pub(crate) struct KMacroSign {
+    name: String,
+    args: Vec<String>,
+    pub logic: KMacroType,
+}
+
+impl KMacroSign {
+    pub fn new(name: &str, args: impl IntoIterator<Item = impl AsRef<str>>, logic: KMacroType) -> Self {
+        let args = args.into_iter().map(|s| s.as_ref().to_owned()).collect::<Vec<String>>();
+        Self {
+            name : name.to_owned(),
+            args,
+            logic,
+        }
+    }
+}
+
+impl std::fmt::Display for KMacroSign {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut inner = self.args.iter().fold(String::new(),|acc, arg| acc + &arg + ",");
+        // This removes last "," character
+        inner.pop();
+        write!(f,"${}({})", self.name, inner)
+    }
+}
+
+#[cfg(feature = "signature")]
+impl From<&KMacroSign> for crate::sigmap::MacroSignature {
+    fn from(bm: &KMacroSign) -> Self {
+        Self {
+            variant: crate::sigmap::MacroVariant::Keyword,
+            name: bm.name.to_owned(),
+            args: bm.args.to_owned(),
+            expr: bm.to_string(),
+        }
+    }
+}
