@@ -1,9 +1,11 @@
 use std::collections::HashMap;
 use std::path::Path;
-use crate::{basic::BasicMacro, keyword_map::KeywordMacro};
+use crate::{basic_map::BasicMacroMap, keyword_map::KeywordMacroMap};
 use crate::error::RadError;
 use crate::utils::Utils;
 use serde::{Deserialize, Serialize};
+#[cfg(feature = "signature")]
+use crate::sigmap::MacroSignature;
 use bincode;
 
 pub type RadResult<T> = Result<T, RadError>;
@@ -15,15 +17,15 @@ pub enum WriteOption {
     Discard,
 }
 
-/// Macro rule of custom macros
+/// Custom macro
 #[derive(Clone, Deserialize, Serialize)]
-pub struct MacroRule{
+pub struct CustomMacro{
     pub name: String,
     pub args: Vec<String>,
     pub body: String,
 }
 
-impl MacroRule {
+impl CustomMacro {
     pub fn new(name: &str, args: &str, body: &str) -> Self {
         // Empty args are no args
         let mut args : Vec<String> = args.split(' ').map(|item| item.to_owned()).collect();
@@ -31,7 +33,7 @@ impl MacroRule {
             args = vec![]
         }
 
-        MacroRule {  
+        CustomMacro {  
             name : name.to_owned(),
             args,
             body : body.to_owned(),
@@ -39,17 +41,53 @@ impl MacroRule {
     }
 }
 
+impl std::fmt::Display for CustomMacro {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut inner = self.args.iter().fold(String::new(),|acc, arg| acc + &arg + ",");
+        // This removes last "," character
+        inner.pop();
+        write!(f,"${}({})", self.name, inner)
+    }
+}
+
+#[cfg(feature = "signature")]
+impl From<&CustomMacro> for crate::sigmap::MacroSignature {
+    fn from(mac: &CustomMacro) -> Self {
+        Self {
+            variant: crate::sigmap::MacroVariant::Custom,
+            name: mac.name.to_owned(),
+            args: mac.args.to_owned(),
+            expr: mac.to_string(),
+        }
+    }
+}
+
+/// Custom macro
+#[derive(Clone)]
+pub struct LocalMacro{
+    pub level: usize,
+    pub name: String,
+    pub body: String,
+}
+
+impl LocalMacro {
+    pub fn new(level: usize,name: String, body: String) -> Self {
+        Self {  level, name, body }
+    }
+}
+
 /// Macro map that stores all kinds of macro informations 
 ///
 /// Included macro types are 
+/// - Keyword macro
 /// - Basic macro
 /// - Custom macro
 /// - Local bound macro
 pub(crate) struct MacroMap {
-    pub keyword: KeywordMacro,
-    pub basic : BasicMacro,
-    pub custom : HashMap<String, MacroRule>,
-    pub local : HashMap<String, String>,
+    pub keyword: KeywordMacroMap,
+    pub basic : BasicMacroMap,
+    pub custom : HashMap<String, CustomMacro>,
+    pub local : HashMap<String, LocalMacro>,
 }
 
 impl MacroMap {
@@ -58,8 +96,8 @@ impl MacroMap {
     /// Keyword macro cannot be empty
     pub fn empty() -> Self {
         Self {
-            keyword: KeywordMacro::new(),
-            basic: BasicMacro::empty(),
+            keyword: KeywordMacroMap::new(),
+            basic: BasicMacroMap::empty(),
             custom: HashMap::new(),
             local: HashMap::new(),
         }
@@ -68,8 +106,8 @@ impl MacroMap {
     /// Creates default map with default basic macros
     pub fn new() -> Self {
         Self { 
-            keyword: KeywordMacro::new(),
-            basic: BasicMacro::new(),
+            keyword: KeywordMacroMap::new(),
+            basic: BasicMacroMap::new(),
             custom: HashMap::new(),
             local: HashMap::new(),
         }
@@ -77,13 +115,18 @@ impl MacroMap {
 
     /// Create a new local macro
     pub fn new_local(&mut self, level: usize,name: &str, value: &str) {
-        self.local.insert(Utils::local_name(level,name), value.to_owned());
+        self.local.insert(Utils::local_name(level,name), LocalMacro::new(level, name.to_owned(), value.to_owned()));
     }
 
     /// Clear all local macros
     pub fn clear_local(&mut self) {
         self.local.clear();
     }
+    
+    /// Retain only local macros that is smaller or equal to current level 
+    pub fn clear_lower_locals(&mut self, current_level: usize) {
+        self.local.retain(|_,mac| mac.level <= current_level);
+    }
 
     pub fn is_keyword(&self, name:&str) -> bool {
         self.keyword.contains(name)
@@ -101,9 +144,9 @@ impl MacroMap {
         name: &str,
         args: &str,
         body: &str,
-    ) -> Result<(),RadError> {
+    ) -> RadResult<()> {
         // Trim all whitespaces and newlines from the string
-        let mac = MacroRule::new(
+        let mac = CustomMacro::new(
             &Utils::trim(name), 
             &Utils::trim(args), 
             body);
@@ -126,27 +169,42 @@ impl MacroMap {
             self.basic.rename(name, target);
         }
         if self.custom.contains_key(name) {
-            let rule = self.custom.remove(name).unwrap();
-            self.custom.insert(target.to_owned(), rule);
+            let custom = self.custom.remove(name).unwrap();
+            self.custom.insert(target.to_owned(), custom);
         }
     }
 
     pub fn append(&mut self, name: &str, target: &str) {
         if self.custom.contains_key(name) {
-            let rule = self.custom.get_mut(name).unwrap();
-            rule.body.push_str(target);
+            let custom = self.custom.get_mut(name).unwrap();
+            custom.body.push_str(target);
         }
     }
 
     pub fn replace(&mut self, name: &str, target: &str) -> bool {
         if self.custom.contains_key(name) {
-            let rule = self.custom.get_mut(name).unwrap();
-            rule.body = target.to_owned();
+            let custom = self.custom.get_mut(name).unwrap();
+            custom.body = target.to_owned();
             true
         } else {
             false
         }
     }
+
+    /// Get macro signatures object
+    #[cfg(feature = "signature")]
+    pub fn get_signatures(&self) -> Vec<MacroSignature> {
+        let key_iter = self.keyword.macros
+            .iter()
+            .map(|(_,sig)| MacroSignature::from(sig));
+        let basic_iter = self.basic.macros
+            .iter()
+            .map(|(_,sig)| MacroSignature::from(sig));
+        let custom_iter = self.custom
+            .iter()
+            .map(|(_,custom)| MacroSignature::from(custom));
+        key_iter.chain(basic_iter).chain(custom_iter).collect()
+    }
 }
 
 /// Struct designed to check unbalanced parenthesis
@@ -178,11 +236,11 @@ impl UnbalancedChecker {
 /// Readable, writeable struct that holds information of custom macros
 #[derive(Serialize, Deserialize)]
 pub struct RuleFile {
-    pub rules : HashMap<String, MacroRule>,
+    pub rules : HashMap<String, CustomMacro>,
 }
 
 impl RuleFile {
-    pub fn new(rules : Option<HashMap<String, MacroRule>>) -> Self {
+    pub fn new(rules : Option<HashMap<String, CustomMacro>>) -> Self {
         if let Some(content) = rules {
             Self {
                 rules: content,
@@ -195,7 +253,7 @@ impl RuleFile {
     }
 
     /// Read from rule file and make it into hash map
-    pub fn melt(&mut self, path : &Path) -> Result<(), RadError> {
+    pub fn melt(&mut self, path : &Path) -> RadResult<()> {
         Utils::is_real_path(path)?;
         let result = bincode::deserialize::<Self>(&std::fs::read(path)?);
         if let Err(_) = result {
@@ -207,7 +265,7 @@ impl RuleFile {
     }
 
     /// Convert custom rules into a single binary file
-    pub(crate) fn freeze(&self, path: &std::path::Path) -> Result<(), RadError> {
+    pub(crate) fn freeze(&self, path: &std::path::Path) -> RadResult<()> {
         let result = bincode::serialize(self);
         if let Err(_) = result {
             Err(RadError::BincodeError(format!("Failed to freeze to the file : {}", path.display())))
@@ -230,6 +288,9 @@ pub(crate) struct MacroFragment {
     pub whole_string: String,
     pub name: String,
     pub args: String,
+    // This yield processed_args information which is not needed for normal operation.
+    #[cfg(feature = "debug")]
+    pub processed_args: String, 
 
     // Macro attributes
     pub pipe: bool,
@@ -244,6 +305,8 @@ impl MacroFragment {
             whole_string : String::new(),
             name : String::new(),
             args : String::new(),
+            #[cfg(feature = "debug")]
+            processed_args : String::new(),
             pipe: false,
             greedy: false,
             yield_literal : false,
@@ -256,6 +319,8 @@ impl MacroFragment {
         self.whole_string.clear();
         self.name.clear();
         self.args.clear();
+        #[cfg(feature = "debug")]
+        self.processed_args.clear();
         self.pipe = false; 
         self.greedy = false; 
         self.yield_literal = false;
@@ -268,6 +333,10 @@ impl MacroFragment {
     pub(crate) fn is_empty(&self) -> bool {
         self.whole_string.len() == 0
     }
+
+    pub(crate) fn has_attribute(&self) -> bool {
+        self.pipe || self.greedy || self.yield_literal || self.trimmed
+    }
 }
 
 /// Comment type 
@@ -290,7 +359,7 @@ pub enum CommentType {
 }
 
 impl CommentType {
-    pub(crate) fn from_str(text : &str) -> Result<Self, RadError> {
+    pub(crate) fn from_str(text : &str) -> RadResult<Self> {
         let comment_type = match text.to_lowercase().as_str() {
             "none"  => Self::None,
             "start" => Self::Start,
