@@ -9,7 +9,7 @@
 //!
 //! # Detailed usage
 //! ```rust
-//! use rad::RadError;
+//! use rad::RadResult;
 //! use rad::Processor;
 //! use rad::MacroType;
 //! use rad::AuthType;
@@ -41,8 +41,6 @@
 //!     .log(true)                                           // Use logging to terminal
 //!     .diff(DiffOption::All)                               // Print diff in final result
 //!     .interactive(true)                                   // Use interactive mode
-//!     // Create unreferenced instance
-//!     .build(); 
 //!
 //! // Comment char and macro char cannot be same 
 //! // Unallowed pattern for the characters are [a-zA-Z1-9\\_\*\^\|\+\(\)=,]
@@ -109,20 +107,22 @@ use std::io::{self, BufReader, Write};
 use std::collections::HashMap;
 use std::fs::{File, OpenOptions};
 use std::path::{ Path , PathBuf};
-use crate::basic::MacroType;
 use crate::closure_map::ClosureMap;
 use crate::error::RadError;
 use crate::logger::{Logger, LoggerLines};
 #[cfg(feature = "debug")]
 use crate::debugger::Debugger;
-use crate::models::{CommentType, MacroFragment, MacroMap, MacroRule, RuleFile, UnbalancedChecker, WriteOption};
+use crate::models::{CommentType, MacroFragment, MacroMap, CustomMacro, RuleFile, UnbalancedChecker, WriteOption, LocalMacro};
 #[cfg(feature = "hook")]
 use crate::hookmap::{HookMap, HookType};
+#[cfg(feature = "signature")]
+use crate::sigmap::SignatureMap;
 use crate::utils::Utils;
 use crate::{RadResult, consts::*};
 use crate::lexor::*;
 use crate::define_parser::DefineParser;
 use crate::arg_parser::{ArgParser, GreedyState};
+use crate::MacroType;
 use regex::Regex;
 use lazy_static::lazy_static;
 
@@ -131,12 +131,12 @@ lazy_static! {
     static ref MAC_NAME : Regex = Regex::new(r#"^[_a-zA-Z]\w*$"#).expect("Failed to create regex expression");
 }
 
-// Methods of processor consists of multiple sections followed as
+// Methods of processor consists of multiple sections followed as <TAG>
 // <BUILDER>            -> Builder pattern related
 // <PROCESS>            -> User functions related
 // <DEBUG>              -> Debug related functions
 // <PARSE>              -> Parse rleated functions
-//     <LEX>            -> sub sectin of parse, this is technically not a lexing but it's named as
+//     <LEX>            -> sub section of parse, this is technically not a lexing but it's named as
 // <MISC>               -> Miscellaenous
 //
 // Find each section's start with <NAME> and find end of section with </NAME>
@@ -260,7 +260,7 @@ impl Processor {
     }
 
     /// Set write option to yield output to the file
-    pub fn write_to_file(&mut self, target_file: Option<impl AsRef<Path>>) -> Result<&mut Self, RadError> {
+    pub fn write_to_file(mut self, target_file: Option<impl AsRef<Path>>) -> RadResult<Self> {
         if let Some(target_file) = target_file {
             let file = OpenOptions::new()
                 .create(true)
@@ -278,7 +278,7 @@ impl Processor {
     }
 
     /// Yield error to the file
-    pub fn error_to_file(&mut self, target_file: Option<impl AsRef<Path>>) -> Result<&mut Self, RadError> {
+    pub fn error_to_file(mut self, target_file: Option<impl AsRef<Path>>) -> RadResult<Self> {
         if let Some(target_file) = target_file {
             let file = OpenOptions::new()
                 .create(true)
@@ -297,7 +297,7 @@ impl Processor {
     }
 
     /// Custom comment character
-    pub fn custom_comment_char(&mut self, character: char) -> Result<&mut Self, RadError> {
+    pub fn custom_comment_char(mut self, character: char) -> RadResult<Self> {
         // check if unallowed character
         if UNALLOWED_CHARS.is_match(&character.to_string()) {
             return Err(RadError::UnallowedChar(format!("\"{}\" is not allowed", character)));
@@ -310,7 +310,7 @@ impl Processor {
     }
 
     /// Custom macro character
-    pub fn custom_macro_char(&mut self, character: char) -> Result<&mut Self, RadError> {
+    pub fn custom_macro_char(mut self, character: char) -> RadResult<Self> {
         if UNALLOWED_CHARS.is_match(&character.to_string()) {
             return Err(RadError::UnallowedChar(format!("\"{}\" is not allowed", character)));
         } else if self.get_comment_char() == character {
@@ -322,7 +322,7 @@ impl Processor {
     }
 
     /// Use unix line ending instead of operating system's default one
-    pub fn unix_new_line(&mut self, use_unix_new_line: bool) -> &mut Self {
+    pub fn unix_new_line(mut self, use_unix_new_line: bool) -> Self {
         if use_unix_new_line {
             self.state.newline = "\n".to_owned();
         }
@@ -330,19 +330,19 @@ impl Processor {
     }
 
     /// Set greedy option
-    pub fn greedy(&mut self, greedy: bool) -> &mut Self {
+    pub fn greedy(mut self, greedy: bool) -> Self {
         self.state.always_greedy = greedy;
         self
     }
 
     /// Set purge option
-    pub fn purge(&mut self, purge: bool) -> &mut Self {
+    pub fn purge(mut self, purge: bool) -> Self {
         self.state.purge = purge;
         self
     }
 
     /// Set lenient
-    pub fn lenient(&mut self, lenient: bool) -> &mut Self {
+    pub fn lenient(mut self, lenient: bool) -> Self {
         self.state.strict = !lenient;
         self
     }
@@ -352,19 +352,19 @@ impl Processor {
     /// This is deprecated and will be removed in 2.0 version.
     /// Use set_comment_type instead.
     #[deprecated(since = "1.2", note = "Comment is deprecated and will be removed in 2.0 version.")]
-    pub fn comment(&mut self, comment: bool) -> &mut Self {
+    pub fn comment(mut self, comment: bool) -> Self {
         if comment { self.state.comment_type = CommentType::Start; }
         self
     }
 
     /// Set comment type
-    pub fn set_comment_type(&mut self, comment_type: CommentType) -> &mut Self {
+    pub fn set_comment_type(mut self, comment_type: CommentType) -> Self {
         self.state.comment_type = comment_type;
         self
     }
 
     /// Set silent option
-    pub fn silent(&mut self, silent: bool) -> &mut Self {
+    pub fn silent(mut self, silent: bool) -> Self {
         if silent {
             self.logger.suppress_warning();
         }
@@ -372,7 +372,7 @@ impl Processor {
     }
 
     /// Set nopanic
-    pub fn nopanic(&mut self, nopanic: bool) -> &mut Self {
+    pub fn nopanic(mut self, nopanic: bool) -> Self {
         if nopanic {
             self.state.nopanic = nopanic;
             self.state.strict = false; 
@@ -381,7 +381,7 @@ impl Processor {
     }
 
     /// Set assertion mode
-    pub fn assert(&mut self, assert: bool) -> &mut Self {
+    pub fn assert(mut self, assert: bool) -> Self {
         if assert { 
             self.logger.assert(); 
             self.write_option = WriteOption::Discard;
@@ -393,34 +393,34 @@ impl Processor {
 
     /// Add debug options
     #[cfg(feature = "debug")]
-    pub fn debug(&mut self, debug: bool) -> &mut Self {
+    pub fn debug(mut self, debug: bool) -> Self {
         self.debugger.debug = debug;
         self
     }
 
     /// Add debug log options
     #[cfg(feature = "debug")]
-    pub fn log(&mut self, log: bool) -> &mut Self {
+    pub fn log(mut self, log: bool) -> Self {
         self.debugger.log = log;
         self
     }
 
     /// Add diff option
     #[cfg(feature = "debug")]
-    pub fn diff(&mut self, diff: DiffOption) -> Result<&mut Self, RadError> {
+    pub fn diff(mut self, diff: DiffOption) -> RadResult<Self> {
         self.debugger.enable_diff(diff)?; 
         Ok(self)
     }
 
     /// Add debug interactive options
     #[cfg(feature = "debug")]
-    pub fn interactive(&mut self, interactive: bool) -> &mut Self {
+    pub fn interactive(mut self, interactive: bool) -> Self {
         if interactive { self.debugger.set_interactive(); }
         self
     }
 
     /// Add custom rules
-    pub fn custom_rules(&mut self, paths: Option<Vec<impl AsRef<Path>>>) -> Result<&mut Self, RadError> {
+    pub fn custom_rules(mut self, paths: Option<Vec<impl AsRef<Path>>>) -> RadResult<Self> {
         if let Some(paths) = paths {
             let mut rule_file = RuleFile::new(None);
             for p in paths.iter() {
@@ -434,7 +434,7 @@ impl Processor {
     }
 
     /// Open authority of processor
-    pub fn allow(&mut self, auth_types : Option<Vec<AuthType>>) -> &mut Self {
+    pub fn allow(mut self, auth_types : Option<Vec<AuthType>>) -> Self {
         if let Some(auth_types) = auth_types {
             for auth in auth_types {
                 self.state.auth_flags.set_state(&auth, AuthState::Open)
@@ -444,7 +444,7 @@ impl Processor {
     }
 
     /// Open authority of processor but yield warning
-    pub fn allow_with_warning(&mut self, auth_types : Option<Vec<AuthType>>) -> &mut Self {
+    pub fn allow_with_warning(mut self, auth_types : Option<Vec<AuthType>>) -> Self {
         if let Some(auth_types) = auth_types {
             for auth in auth_types {
                 self.state.auth_flags.set_state(&auth, AuthState::Warn)
@@ -454,7 +454,7 @@ impl Processor {
     }
 
     /// Discard output
-    pub fn discard(&mut self, discard: bool) -> &mut Self {
+    pub fn discard(mut self, discard: bool) -> Self {
         if discard {
             self.write_option = WriteOption::Discard;
         }
@@ -462,8 +462,9 @@ impl Processor {
     }
 
     /// Creates an unreferenced instance of processor
-    pub fn build(&mut self) -> Self {
-        std::mem::replace(self, Processor::new())
+    #[deprecated(since = "1.3", note = "Build method is deprecated in favor of more ergonomic builder pattern. It will be removed in 2.0")]
+    pub fn build(self) -> Self {
+        self
     }
 
     // </BUILDER>
@@ -474,10 +475,16 @@ impl Processor {
     // Processing methods
     // <PROCESS>
     //
+    
+    /// Get macro signatrue map
+    #[cfg(feature = "signature")]
+    pub fn get_signature_map(&self) -> RadResult<SignatureMap> {
+        Ok(SignatureMap::new(self.map.get_signatures()))
+    }
 
     /// Print current permission status
     #[allow(dead_code)]
-    pub fn print_permission(&mut self) -> Result<(), RadError> {
+    pub fn print_permission(&mut self) -> RadResult<()> {
         if let Some(status) = self.state.auth_flags.get_status_string() {
             let mut status_with_header = String::from("Permission granted");
             status_with_header.push_str(&status);
@@ -488,7 +495,7 @@ impl Processor {
 
     /// Print the result of a processing
     #[allow(dead_code)]
-    pub fn print_result(&mut self) -> Result<(), RadError> {
+    pub fn print_result(&mut self) -> RadResult<()> {
         self.logger.print_result()?;
 
         #[cfg(feature = "debug")]
@@ -500,7 +507,7 @@ impl Processor {
     /// Freeze to single file
     ///
     /// Frozen file is a bincode encoded binary format file.
-    pub fn freeze_to_file(&self, path: impl AsRef<Path>) -> Result<(), RadError> {
+    pub fn freeze_to_file(&self, path: impl AsRef<Path>) -> RadResult<()> {
         // File path validity is checked by freeze method
         RuleFile::new(Some(self.map.custom.clone())).freeze(path.as_ref())?;
         Ok(())
@@ -551,7 +558,7 @@ impl Processor {
     /// ```rust
     /// processor.add_custom_rules(vec![("macro_name","macro_arg1 macro_arg2","macro_body=$macro_arg1()")]);
     /// ```
-    pub fn add_custom_rules(&mut self, rules: Vec<(impl AsRef<str>,&str,&str)>) -> Result<(), RadError> {
+    pub fn add_custom_rules(&mut self, rules: Vec<(impl AsRef<str>,&str,&str)>) -> RadResult<()> {
         for (name,args,body) in rules {
             let name = name.as_ref();
             if !MAC_NAME.is_match(name) {
@@ -559,7 +566,7 @@ impl Processor {
             }
             self.map.custom.insert(
                 name.to_owned(), 
-                MacroRule { 
+                CustomMacro { 
                     name: name.to_owned(),
                     args: args.split(' ').map(|s| s.to_owned()).collect::<Vec<String>>(),
                     body: body.to_owned()
@@ -588,7 +595,7 @@ impl Processor {
             }
             self.map.custom.insert(
                 name.to_owned(), 
-                MacroRule { 
+                CustomMacro { 
                     name: name.to_owned(),
                     args: vec![],
                     body: body.to_owned()
@@ -627,7 +634,7 @@ impl Processor {
     }
 
     /// Read from string
-    pub fn from_string(&mut self, content: &str) -> Result<(), RadError> {
+    pub fn from_string(&mut self, content: &str) -> RadResult<()> {
         // Set name as string
         self.set_input("String")?;
 
@@ -640,7 +647,7 @@ impl Processor {
     ///
     /// If debug mode is enabled this, doesn't read stdin line by line but by chunk because user
     /// input is also a standard input and processor cannot distinguish the two
-    pub fn from_stdin(&mut self) -> Result<(), RadError> {
+    pub fn from_stdin(&mut self) -> RadResult<()> {
         let stdin = io::stdin();
 
         // Early return if debug
@@ -660,7 +667,7 @@ impl Processor {
     }
 
     /// Process contents from a file
-    pub fn from_file(&mut self, path :impl AsRef<Path>) -> Result<(), RadError> {
+    pub fn from_file(&mut self, path :impl AsRef<Path>) -> RadResult<()> {
         // Sandboxed environment, backup
         let backup = if self.state.sandbox { Some(self.backup()) } else { None };
         // Set file as name of given path
@@ -672,7 +679,7 @@ impl Processor {
         Ok(())
     }
 
-    pub(crate) fn from_file_as_chunk(&mut self, path :impl AsRef<Path>) -> Result<Option<String>, RadError> {
+    pub(crate) fn from_file_as_chunk(&mut self, path :impl AsRef<Path>) -> RadResult<Option<String>> {
         // Sandboxed environment, backup
         let backup = if self.state.sandbox { Some(self.backup()) } else { None };
         // Set file as name of given path
@@ -685,7 +692,7 @@ impl Processor {
     }
 
     /// Internal method for processing buffers line by line
-    fn from_buffer(&mut self,buffer: &mut impl std::io::BufRead, backup: Option<SandboxBackup>, use_container: bool) -> Result<Option<String>, RadError> {
+    fn from_buffer(&mut self,buffer: &mut impl std::io::BufRead, backup: Option<SandboxBackup>, use_container: bool) -> RadResult<Option<String>> {
         let mut line_iter = Utils::full_lines(buffer).peekable();
         let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
@@ -781,7 +788,7 @@ impl Processor {
 
     /// Check if debug macro should be executed
     #[cfg(feature = "debug")]
-    fn check_debug_macro(&mut self, frag: &mut MacroFragment, level: usize) -> Result<(), RadError> {
+    fn check_debug_macro(&mut self, frag: &mut MacroFragment, level: usize) -> RadResult<()> {
         // Early return if not in a debug mode
         if !self.is_debug() { return Ok(()); }
 
@@ -809,7 +816,7 @@ impl Processor {
     ///
     /// This parses given input as line by line with an iterator of lines including trailing new
     /// line chracter.
-    fn parse_line(&mut self, lines :&mut impl std::iter::Iterator<Item = std::io::Result<String>>, lexor : &mut Lexor ,frag : &mut MacroFragment) -> Result<ParseResult, RadError> {
+    fn parse_line(&mut self, lines :&mut impl std::iter::Iterator<Item = std::io::Result<String>>, lexor : &mut Lexor ,frag : &mut MacroFragment) -> RadResult<ParseResult> {
         self.logger.add_line_number();
         if let Some(line) = lines.next() {
             let line = line?;
@@ -844,7 +851,7 @@ impl Processor {
     } // parse_line end
 
     /// Parse chunk args by separating it into lines which implements BufRead
-    pub(crate) fn parse_chunk_args(&mut self, level: usize, _caller: &str, chunk: &str) -> Result<String, RadError> {
+    pub(crate) fn parse_chunk_args(&mut self, level: usize, _caller: &str, chunk: &str) -> RadResult<String> {
         let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
         let mut result = String::new();
@@ -878,7 +885,7 @@ impl Processor {
     /// 
     /// In contrast to parse_chunk_lines, parse_chunk doesn't create lines iterator but parses the
     /// chunk as a single entity or line.
-    fn parse_chunk_body(&mut self, level: usize, caller: &str, chunk: &str) -> Result<String, RadError> {
+    fn parse_chunk_body(&mut self, level: usize, caller: &str, chunk: &str) -> RadResult<String> {
         let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
         let backup = self.logger.backup_lines();
@@ -894,7 +901,7 @@ impl Processor {
     /// Parse a given line
     ///
     /// This calles lexor.lex to validate characters and decides next behaviour
-    fn parse(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, line: &str, level: usize, caller: &str) -> Result<String, RadError> {
+    fn parse(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, line: &str, level: usize, caller: &str) -> RadResult<String> {
         // Initiate values
         // Reset character number
         self.logger.reset_char_number();
@@ -943,7 +950,7 @@ impl Processor {
                     self.lex_branch_add_to_remainder(ch, &mut remainder)?;
                 }
                 LexResult::AddToFrag(cursor) => {
-                    self.lex_branch_add_to_frag(ch, frag, cursor);
+                    self.lex_branch_add_to_frag(ch, frag, cursor)?;
                 }
                 LexResult::EndFrag => {
                     self.lex_branch_end_frag(ch,frag,&mut remainder, lexor, level, caller)?;
@@ -991,9 +998,11 @@ impl Processor {
     /// - Local bound macro
     /// - Custom macro
     /// - Basic macro
-    fn evaluate(&mut self,level: usize, caller: &str, name: &str, raw_args: &str, greedy: bool) -> Result<EvalResult, RadError> {
+    fn evaluate(&mut self,level: usize, caller: &str, frag : &mut MacroFragment) -> RadResult<EvalResult> {
         // Increase level to represent nestedness
         let level = level + 1;
+        let (name, raw_args) = (&frag.name, &frag.args);
+        let greedy = frag.greedy || self.state.always_greedy;
 
         let mut args : String = raw_args.to_owned();
         // Preprocess only when macro is not a keyword macro
@@ -1001,6 +1010,10 @@ impl Processor {
             // This parses and processes arguments
             // and macro should be evaluated after
             args = self.parse_chunk_args(level, name, raw_args)?;
+
+            // Also update original arguments for better debugging
+            #[cfg(feature = "debug")]
+            { frag.processed_args = args.clone(); }
         }
 
         // Possibly inifinite loop so warn user
@@ -1010,7 +1023,7 @@ impl Processor {
 
         // Find keyword macro
         if self.map.is_keyword(name) {
-            let func = self.map.keyword.get(name).unwrap();
+            let func = self.map.keyword.get_func(name).unwrap();
             let final_result = func(&args, level,self)?;
             return Ok(EvalResult::Eval(final_result));
         }
@@ -1020,7 +1033,7 @@ impl Processor {
         let mut temp_level = level;
         while temp_level > 0 {
             if let Some(local) = self.map.local.get(&Utils::local_name(temp_level, &name)) {
-                return Ok(EvalResult::Eval(Some(local.to_owned())));
+                return Ok(EvalResult::Eval(Some(local.body.to_owned())));
             } 
             temp_level = temp_level - 1;
         }
@@ -1037,7 +1050,7 @@ impl Processor {
         // Find basic macro
         else if self.map.basic.contains(&name) {
             // Func always exists, because contains succeeded.
-            let func = self.map.basic.get(name).unwrap();
+            let func = self.map.basic.get_func(name).unwrap();
             let final_result = func(&args, greedy, self)?;
             return Ok(EvalResult::Eval(final_result));
         } 
@@ -1055,7 +1068,7 @@ impl Processor {
     /// Invoke a custom rule and get a result
     ///
     /// Invoke rule evaluates body of macro rule because body is not evaluated on register process
-    fn invoke_rule(&mut self,level: usize ,name: &str, arg_values: &str, greedy: bool) -> Result<Option<String>, RadError> {
+    fn invoke_rule(&mut self,level: usize ,name: &str, arg_values: &str, greedy: bool) -> RadResult<Option<String>> {
         // Get rule
         // Invoke is called only when key exists, thus unwrap is safe
         let rule = self.map.custom.get(name).unwrap().clone();
@@ -1077,13 +1090,16 @@ impl Processor {
         // Process the rule body
         let result = self.parse_chunk_body(level, &name, &rule.body)?;
 
+        // Clear lower locals to prevent local collisions
+        self.map.clear_lower_locals(level);
+
         Ok(Some(result))
     }
 
     /// Add custom rule to macro map
     ///
     /// This doesn't clear fragment
-    fn add_rule(&mut self, frag: &MacroFragment, remainder: &mut String) -> Result<(), RadError> {
+    fn add_rule(&mut self, frag: &MacroFragment, remainder: &mut String) -> RadResult<()> {
         if let Some((name,args,body)) = self.defparser.parse_define(&frag.args) {
 
             // Strict mode
@@ -1105,7 +1121,7 @@ impl Processor {
     }
 
     /// Write text to either file or standard output according to processor's write option
-    fn write_to(&mut self, content: &str, container: &mut Option<String>) -> Result<(), RadError> {
+    fn write_to(&mut self, content: &str, container: &mut Option<String>) -> RadResult<()> {
         // Don't try to write empty string, because it's a waste
         if content.len() == 0 { return Ok(()); }
 
@@ -1160,7 +1176,7 @@ impl Processor {
         }
     }
 
-    fn lex_branch_start_frag(&mut self, ch: char,frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor) -> Result<(), RadError> {
+    fn lex_branch_start_frag(&mut self, ch: char,frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor) -> RadResult<()> {
         #[cfg(feature = "debug")]
         self.debugger.user_input_before_macro(&frag, &mut self.logger)?;
 
@@ -1189,7 +1205,7 @@ impl Processor {
         }
     }
 
-    fn lex_branch_add_to_remainder(&mut self, ch: char,remainder: &mut String) -> Result<(), RadError> {
+    fn lex_branch_add_to_remainder(&mut self, ch: char,remainder: &mut String) -> RadResult<()> {
         if !self.checker.check(ch) && !self.state.paused {
             self.logger.freeze_number();
             self.log_warning("Unbalanced parenthesis detected.")?;
@@ -1199,7 +1215,7 @@ impl Processor {
         Ok(())
     }
 
-    fn lex_branch_add_to_frag(&mut self, ch: char,frag: &mut MacroFragment, cursor: Cursor) {
+    fn lex_branch_add_to_frag(&mut self, ch: char,frag: &mut MacroFragment, cursor: Cursor) -> RadResult<()> {
         match cursor{
             Cursor::Name => {
                 if frag.name.len() == 0 {
@@ -1210,7 +1226,13 @@ impl Processor {
                     '+' => frag.greedy = true,
                     '*' => frag.yield_literal = true,
                     '^' => frag.trimmed = true,
-                    _ => frag.name.push(ch) 
+                    _ => {
+                        if frag.has_attribute() {
+                            self.log_error("Received non attribute character after macro attributes which is not allowed")?;
+                            return Err(RadError::InvalidMacroName("Invalid macro attribute".to_owned()));
+                        }
+                        frag.name.push(ch);
+                    }
                 }
             },
             Cursor::Arg => {
@@ -1219,9 +1241,10 @@ impl Processor {
             _ => unreachable!(),
         } 
         frag.whole_string.push(ch);
+        Ok(())
     }
 
-    fn lex_branch_end_frag(&mut self, ch: char, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize, caller: &str) -> Result<(), RadError> {
+    fn lex_branch_end_frag(&mut self, ch: char, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize, caller: &str) -> RadResult<()> {
         // Push character to whole string anyway
         frag.whole_string.push(ch);
 
@@ -1234,7 +1257,7 @@ impl Processor {
     }
 
     // Level is necessary for debug feature
-    fn lex_branch_end_frag_define(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, remainder: &mut String, level: usize) -> Result<(), RadError> {
+    fn lex_branch_end_frag_define(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, remainder: &mut String, level: usize) -> RadResult<()> {
         self.add_rule(frag, remainder)?;
         lexor.escape_next_newline();
         #[cfg(feature = "debug")]
@@ -1247,7 +1270,7 @@ impl Processor {
     // TODO
     // This should be renamed because it is now, not only used by lex branches
     // but also used by external logic functions such as hook macro executions.
-    fn lex_branch_end_invoke(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, remainder: &mut String, level: usize, caller: &str) -> Result<(), RadError> {
+    fn lex_branch_end_invoke(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, remainder: &mut String, level: usize, caller: &str) -> RadResult<()> {
         // Name is empty
         if frag.name.len() == 0 {
             self.log_error("Name is empty")?;
@@ -1273,7 +1296,9 @@ impl Processor {
         }
 
         // Try to evaluate
-        let evaluation_result = self.evaluate(level, caller, &frag.name, &frag.args, frag.greedy || self.state.always_greedy);
+        // let evaluation_result = self.evaluate(level, caller, &frag.name, &frag.args, frag.greedy || self.state.always_greedy);
+
+        let evaluation_result = self.evaluate(level, caller, frag);
 
         match evaluation_result {
             // If panicked, this means unrecoverable error occured.
@@ -1289,7 +1314,7 @@ impl Processor {
         Ok(())
     }
 
-    fn lex_branch_end_frag_eval_result_error(&mut self, error : RadError) -> Result<(), RadError> {
+    fn lex_branch_end_frag_eval_result_error(&mut self, error : RadError) -> RadResult<()> {
         // this is equlvalent to conceptual if let not pattern
         if let RadError::Panic = error{
             // Do nothing
@@ -1307,7 +1332,7 @@ impl Processor {
     }
 
     // Level is needed for feature debug & hook codes
-    fn lex_branch_end_frag_eval_result_ok(&mut self, variant : EvalResult, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize) -> Result<(), RadError> {
+    fn lex_branch_end_frag_eval_result_ok(&mut self, variant : EvalResult, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize) -> RadResult<()> {
         match variant {
             EvalResult::Eval(content) => {
 
@@ -1452,6 +1477,12 @@ impl Processor {
             .unwrap());
     }
 
+    /// Set debug flag
+    #[cfg(feature = "debug")]
+    pub(crate) fn set_debug(&mut self, debug: bool)  {
+        self.debugger.debug = debug;
+    }
+
     /// Turn on sandbox
     ///
     /// This is an explicit state change method for non-processor module's usage
@@ -1482,7 +1513,7 @@ impl Processor {
     }
 
     /// Recover backup information into the processor
-    fn recover(&mut self, backup: SandboxBackup) -> Result<(), RadError> {
+    fn recover(&mut self, backup: SandboxBackup) -> RadResult<()> {
         // NOTE ::: Set file should come first becuase set_file override line number and character number
         self.logger.set_file(&backup.current_input);
         self.state.current_input = backup.current_input;
@@ -1494,26 +1525,26 @@ impl Processor {
         Ok(())
     }
 
-    pub(crate) fn track_assertion(&mut self, success: bool) -> Result<(), RadError> {
+    pub(crate) fn track_assertion(&mut self, success: bool) -> RadResult<()> {
         self.logger.alog(success)?;
         Ok(())
     }
 
     /// Log error
-    pub(crate) fn log_error(&mut self, log : &str) -> Result<(), RadError> {
+    pub(crate) fn log_error(&mut self, log : &str) -> RadResult<()> {
         self.logger.elog(log)?;
         Ok(())
     }
 
     /// Log warning
-    pub(crate) fn log_warning(&mut self, log : &str) -> Result<(), RadError> {
+    pub(crate) fn log_warning(&mut self, log : &str) -> RadResult<()> {
         self.logger.wlog(log)?;
         Ok(())
     }
 
     // This is not a backup but fresh set of file information
     /// Set(update) current processing file information
-    fn set_file(&mut self, file: &str) -> Result<(), RadError> {
+    fn set_file(&mut self, file: &str) -> RadResult<()> {
         let path = Path::new(file);
         if !path.exists() {
             Err(RadError::InvalidCommandOption(format!("File, \"{}\" doesn't exist, therefore cannot be read by r4d.", path.display())))
@@ -1526,7 +1557,7 @@ impl Processor {
     }
 
     /// Set some useful env values
-    fn set_file_env(&self, file: &str) -> Result<(), RadError> {
+    fn set_file_env(&self, file: &str) -> RadResult<()> {
         let path = Path::new(file);
         std::env::set_var("RAD_FILE", file);
         std::env::set_var("RAD_FILE_DIR", std::fs::canonicalize(path)?.parent().unwrap());
@@ -1536,7 +1567,7 @@ impl Processor {
     /// Set input as string not as &path
     /// 
     /// This is conceptualy identical to set_file but doesn't validate if given input is existent
-    fn set_input(&mut self, input: &str) -> Result<(), RadError> {
+    fn set_input(&mut self, input: &str) -> RadResult<()> {
         self.state.current_input = input.to_owned();
         self.logger.set_file(input);
         Ok(())
@@ -1578,7 +1609,7 @@ enum ParseResult {
 /// e.g. when include macro is called, outer file's information is not helpful at all.
 struct SandboxBackup {
     current_input: String,
-    local_macro_map: HashMap<String,String>,
+    local_macro_map: HashMap<String,LocalMacro>,
     logger_lines: LoggerLines,
 }
 
